@@ -53,12 +53,10 @@ my %REFINABLE_ARGS = (
   map {(-$_ => \&_just_store_arg)} qw/order_by        group_by  having    for
                                       union union_all intersect except    minus
                                       result_as       post_SQL  pre_exec  post_exec  post_bless
-                                      limit           offset    page_size page_index
+                                      limit           offset    page_size page_index as
                                       column_types    prepare_attrs       dbi_prepare_method
-                                      _left_cols      where_on            join_with_USING
-                                      sql_abstract/,
+                                      where_on        join_with_USING     sql_abstract/,
  );
-# TOCHECK : don't remember why we need _left_cols. What is it used for ?
 
 
 #----------------------------------------------------------------------
@@ -233,7 +231,7 @@ sub sqlize {
   my @args_to_copy = qw/-columns -where
                         -union -union_all -intersect -except -minus
                         -order_by -group_by -having
-                        -limit -offset -page_size -page_index/;
+                        -limit -offset -page_size -page_index -as/;
   my %sqla_args = (-from         => clone($self->source->db_from),
                    -want_details => 1);
   defined $args->{$_} and $sqla_args{$_} = $args->{$_} for @args_to_copy;
@@ -348,14 +346,15 @@ sub prepare {
   # log the statement and bind values
   $self->schema->_debug("PREPARE $self->{sql} / @{$self->{bound_params}}");
 
-  # call the database
+  # assemble stuff for calling the database
   my $dbh          = $self->schema->dbh or croak "Schema has no dbh";
-  my $method       = $self->{args}{-dbi_prepare_method}
-                  || $self->schema->dbi_prepare_method;
+  my $method       = $self->{args}{-dbi_prepare_method}  || $self->schema->dbi_prepare_method;
   my @prepare_args = ($self->{sql});
   if (my $prepare_attrs = $self->{args}{-prepare_attrs}) {
     push @prepare_args, $prepare_attrs;
   }
+
+  # call the database
   $self->{sth}  = $dbh->$method(@prepare_args);
 
   # new status and return
@@ -364,6 +363,7 @@ sub prepare {
 }
 
 
+
 sub sth {
   my ($self) = @_;
 
